"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const CryptoJs = require("crypto-js");
const Https = require("https");
const SignalR = require("signalr-client");
const url_1 = require("url");
const Market_1 = require("./model/Market");
const Currency_1 = require("./model/Currency");
const Ticker_1 = require("./model/Ticker");
const MarketSummary_1 = require("./model/MarketSummary");
const Order_1 = require("./model/Order");
const OrderBookType_1 = require("./enum/OrderBookType");
const Trade_1 = require("./model/Trade");
const OrderType_1 = require("./enum/OrderType");
const OpenOrder_1 = require("./model/OpenOrder");
const Balance_1 = require("./model/Balance");
const ExchangeStateUpdate_1 = require("./model/ExchangeStateUpdate");
const util_1 = require("util");
const ResponseParsingError_1 = require("./error/ResponseParsingError");
const WebSocketError_1 = require("./error/WebSocketError");
/**
 * Represents a single Bittrex API client.
 */
class BittrexApiClient {
    /**
     * Initializes a new Bittrex API client.
     *
     * @param apiKey    The personal account API key.
     * @param apiSecret The personal account API secret.
     */
    constructor(apiKey, apiSecret) {
        if (util_1.isNullOrUndefined(apiKey) || util_1.isNullOrUndefined(apiSecret)) {
            throw new Error("API key and API secret cannot be undefined or null");
        }
        this.apiKey = apiKey;
        this.apiSecret = apiSecret;
    }
    /**
     * Interface to the "public/getmarkets" Bittrex's API operation.
     *
     * @returns Either a promise of a market, or a market
     *          if using the await construct.
     */
    getMarkets() {
        return __awaiter(this, void 0, void 0, function* () {
            let marketsJson = yield this.makeRequest("public/getmarkets");
            let markets = [];
            for (let marketJson of marketsJson) {
                markets.push(new Market_1.Market(marketJson));
            }
            return markets;
        });
    }
    /**
     * Interface to the "public/getcurrencies" Bittrex's API operation.
     *
     * @returns Either a promise of a currency array, or a currency array
     *          if using the await construct.
     */
    getCurrencies() {
        return __awaiter(this, void 0, void 0, function* () {
            let currenciesJson = yield this.makeRequest("public/getcurrencies");
            let currencies = [];
            for (let currencyJson of currenciesJson) {
                currencies.push(new Currency_1.Currency(currencyJson));
            }
            return currencies;
        });
    }
    /**
     * Interface to the "public/getticker" Bittrex's API operation.
     *
     * @param   market The market of which we would like
     *                 to retrieve the ticker.
     * @returns Either a promise of a ticker, or a ticker
     *          if using the await construct.
     */
    getTicker(market) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Ticker_1.Ticker(yield this.makeRequest("public/getticker", ["market", market]));
        });
    }
    /**
     * Interface to the "public/getmarketsummaries" Bittrex's API operation.
     *
     * @returns Either a promise of a market array, or a market array
     *          if using the await construct.
     */
    getMarketSummaries() {
        return __awaiter(this, void 0, void 0, function* () {
            let marketSummariesJson = yield this.makeRequest("public/getmarketsummaries");
            let marketSummaries = [];
            for (let marketSummaryJson of marketSummariesJson) {
                marketSummaries.push(new MarketSummary_1.MarketSummary(marketSummaryJson));
            }
            return marketSummaries;
        });
    }
    /**
     * Interface to the "public/getmarketsummary" Bittrex's API operation.
     *
     * @param   market The market of which we would like
     *                 to retrieve the summary.
     * @returns Either a promise of a market summary, or a market summary
     *          if using the await construct.
     */
    getMarketSummary(market) {
        return __awaiter(this, void 0, void 0, function* () {
            return new MarketSummary_1.MarketSummary((yield this.makeRequest("public/getmarketsummary", ["market", market]))[0]);
        });
    }
    /**
     * Interface to the "public/getorderbook" Bittrex's API operation.
     *
     * @param   market The market of which we would like
     *                 to retrieve the order book.
     * @param   type   The type of the order book that we want to
     *                 retrieve, depending on if we want only the
     *                 buys, sells, or both.
     * @returns Either a promise of an order book, or an order book
     *          if using the await construct.
     */
    getOrderBook(market, type) {
        return __awaiter(this, void 0, void 0, function* () {
            let ordersJson = yield this.makeRequest("public/getorderbook", ["market", market], ["type", OrderBookType_1.OrderBookType[type]]);
            let orders = [];
            if (type === OrderBookType_1.OrderBookType.BOTH) {
                for (let buyOrderJson of ordersJson.buy) {
                    orders.push(new Order_1.Order(buyOrderJson, OrderType_1.OrderType.BUY));
                }
                for (let sellOrderJson of ordersJson.sell) {
                    orders.push(new Order_1.Order(sellOrderJson, OrderType_1.OrderType.SELL));
                }
                return orders;
            }
            for (let orderJson of ordersJson) {
                ordersJson.push(new Order_1.Order(orderJson, type === OrderBookType_1.OrderBookType.BUY ? OrderType_1.OrderType.BUY : OrderType_1.OrderType.SELL));
            }
            return orders;
        });
    }
    /**
     * Interface to the "public/getmarkethistory" Bittrex's API operation.
     *
     * @param   market The market of which we would like
     *                 to retrieve the market history.
     * @returns Either a promise of a market history, or a market history
     *          if using the await construct.
     */
    getMarketHistory(market) {
        return __awaiter(this, void 0, void 0, function* () {
            let tradesJson = yield this.makeRequest("/public/getmarkethistory", ["market", market]);
            let trades = [];
            for (let tradeJson of tradesJson) {
                trades.push(new Trade_1.Trade(tradeJson));
            }
            return trades;
        });
    }
    /**
     * Interface to the "public/buylimit" Bittrex's API operation.
     *
     * @param   market   The market on which we would like to buy.
     * @param   quantity The quantity that we would like to buy.
     * @param   rate     The price at which we would like to buy.
     * @returns Either a promise of the placed order's ID, or the placed
     *          order's ID if using the await construct.
     */
    buyWithLimit(market, quantity, rate) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.makeRequest("/market/buylimit", ["market", market], ["quantity", quantity.toString()], ["rate", rate.toString()])).uuid;
        });
    }
    /**
     * Interface to the "public/selllimit" Bittrex's API operation.
     *
     * @param   market   The market on which we would like to sell.
     * @param   quantity The quantity that we would like to sell.
     * @param   rate     The price at which we would like to sell.
     * @returns Either a promise of the placed order's ID, or the placed
     *          order's ID if using the await construct.
     */
    sellWithLimit(market, quantity, rate) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.makeRequest("/market/selllimit", ["market", market], ["quantity", quantity.toString()], ["rate", rate.toString()])).uuid;
        });
    }
    /**
     * Interface to the "public/cancelorder" Bittrex's API operation.
     *
     * @param   orderId The ID of the order we would like to cancel.
     * @returns True if the operation resulted in a success, throws
     *          otherwise.
     */
    cancelOrder(orderId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.makeRequest("/market/cancelorder", ["uuid", orderId]);
            return true;
        });
    }
    /**
     * Interface to the "public/getopenorders" Bittrex's API operation.
     *
     * @param   market The market of which we would like to retrieve
     *                 the open orders.
     * @returns Either a promise of an open order array, or an open
     *          order array if using the await construct.
     */
    getOpenOrders(market) {
        return __awaiter(this, void 0, void 0, function* () {
            let openOrdersJson = yield this.makeRequest("/market/getopenorders", ["market", market]);
            let openOrders = [];
            for (let openOrderJson of openOrdersJson) {
                openOrders.push(new OpenOrder_1.OpenOrder(openOrderJson));
            }
            return openOrders;
        });
    }
    /**
     * Interface to the "public/getbalances" Bittrex's API operation.
     *
     * @returns Either a promise of a balance array, or a balance
     *          array if using the await construct.
     */
    getBalances() {
        return __awaiter(this, void 0, void 0, function* () {
            let balancesJson = yield this.makeRequest("/account/getbalances");
            let balances = [];
            for (let balanceJson of balancesJson) {
                balances.push(new Balance_1.Balance(balanceJson));
            }
            return balances;
        });
    }
    /**
     * Interface to the "public/getbalance" Bittrex's API operation.
     *
     * @param currency The currency of which we would like to retrieve
     *                 the balance.
     * @returns Either a promise of a balance, or a balance if using
     *          the await construct.
     */
    getBalance(currency) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Balance_1.Balance(yield this.makeRequest("/account/getbalance", ["currency", currency]));
        });
    }
    /**
     * Interface to the "public/getdepositaddress" Bittrex's API operation.
     *
     * @param currency The currency of which we would like to retrieve
     *                 the deposit address.
     * @returns Either a promise of a deposit address, or a deposit
     *          address if using the await construct.
     */
    getDepositAddress(currency) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.makeRequest("/account/getdepositaddress", ["currency", currency])).Address;
        });
    }
    /**
     * Interface to the "public/getdepositaddress" Bittrex's API operation.
     *
     * @param currency  The currency which we would like to withdraw.
     * @param quantity  The quantity which we would like to withdraw.
     * @param address   The address to which we would like to withdraw.
     * @param paymentId Optional parameter used for CryptoNotes/BitShareX/Nxt.
     *
     * @returns Either a promise of a withdrawal ID, or a withdrawal ID
     *          if using the await construct.
     */
    withdraw(currency, quantity, address, paymentId) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.makeRequest("/account/withdraw", ["currency", currency], ["quantity", quantity.toString()], ["address", address], ["paymentId", paymentId])).uuid;
        });
    }
    /**
     * Interface to the Bittrex's API websockets system.
     *
     * @param watchableMarkets The markets of which we would like
     *                         to receive constant updates.
     * @param callback A callback function invoked as soon as new
     *                updates about the watched markets are received
     *                from Bittrex.
     */
    getExchangeStateUpdatesStream(watchableMarkets, callback) {
        let websocketClient = new SignalR.client("wss://socket.bittrex.com/signalr", ["CoreHub"]);
        websocketClient.serviceHandlers.reconnecting = () => {
            return false;
        };
        websocketClient.serviceHandlers.onerror = (error) => {
            throw new WebSocketError_1.WebSocketError(error);
        };
        websocketClient.serviceHandlers.connected = () => {
            for (let watchableMarket of watchableMarkets) {
                websocketClient.call("CoreHub", "SubscribeToExchangeDeltas", watchableMarket);
            }
        };
        websocketClient.serviceHandlers.messageReceived = (messageJson) => {
            if (messageJson.type !== "utf8") {
                return;
            }
            try {
                messageJson = JSON.parse(messageJson.utf8Data);
            }
            catch (error) {
                throw new ResponseParsingError_1.ResponseParsingError(`An error occurred parsing Bittrex's response. The response was: ${messageJson}`);
            }
            let updatesJson = messageJson.M;
            if (updatesJson === undefined || updatesJson.length === 0) {
                return;
            }
            let exchangeStateUpdates = [];
            for (let updateJson of updatesJson) {
                if (updateJson.M !== "updateExchangeState") {
                    continue;
                }
                for (let exchangeStateUpdateJson of updateJson.A) {
                    exchangeStateUpdates.push(new ExchangeStateUpdate_1.ExchangeStateUpdate(exchangeStateUpdateJson));
                }
            }
            if (exchangeStateUpdates.length === 0) {
                return;
            }
            callback(exchangeStateUpdates);
        };
    }
    /**
     * Utility method that sends a request to the Bittrex's API, handling the
     * authentication through the API key and API secret given when instantiating
     * the client itself.
     *
     * @param operation The Bittrex's API operation that we would like to call.
     * @param parameters The parameters which the operation takes in.
     *
     * @returns Either the promise of the Bittrex's API JSON response, or the
     *          JSON response if using the await construct.
     */
    makeRequest(operation, ...parameters) {
        return __awaiter(this, void 0, void 0, function* () {
            let apiEndpointUrl = new url_1.URL("/api/v1.1/" + operation, "https://www.bittrex.com/");
            apiEndpointUrl.searchParams.append("apikey", this.apiKey);
            apiEndpointUrl.searchParams.append("nonce", new Date().getTime().toString());
            for (let parameter of parameters) {
                if (parameter === undefined || parameter === null) {
                    continue;
                }
                apiEndpointUrl.searchParams.append(parameter[0], parameter[1]);
            }
            let apiSign = CryptoJs.HmacSHA512(apiEndpointUrl.toString(), this.apiSecret);
            return new Promise((fulfill, reject) => {
                let clientRequest = Https.request(apiEndpointUrl, (bittrexResponse) => {
                    bittrexResponse.on("data", (bittrexData) => {
                        try {
                            bittrexData = JSON.parse(bittrexData.toString());
                        }
                        catch (error) {
                            throw new ResponseParsingError_1.ResponseParsingError(`An error occurred parsing Bittrex's response. The response was: ${bittrexData.toString()}`);
                        }
                        if (bittrexData.success) {
                            fulfill(bittrexData.result);
                        }
                        else {
                            reject(bittrexData.message);
                        }
                    });
                });
                clientRequest.setHeader("apisign", apiSign);
                clientRequest.end();
            });
        });
    }
}
exports.BittrexApiClient = BittrexApiClient;
//# sourceMappingURL=BittrexApiClient.js.map